var assert = require('assert');
var elliptic = require('./');

console.time('curve load');
var curve = elliptic.nist.secp256k1;
assert(curve);
console.timeEnd('curve load');

var ecdsa = new elliptic.ecdsa(curve);
console.time('gen key');
var keys = ecdsa.genKeyPair();
console.timeEnd('gen key');
console.log(keys);
var msg = new elliptic.bn('deadbeef', 16);

console.time('validate');
var v = ecdsa.validateKey(keys.pub);
assert(v.result, v.reason);
console.timeEnd('validate');

// Sign and verify
console.time('sign');
var signature = ecdsa.sign(msg, keys.priv);
console.timeEnd('sign');
console.time('verify');
assert(ecdsa.verify(msg, signature, keys.pub));
console.timeEnd('verify');

// Wrong public key
console.time('get key 2');
var keys = ecdsa.genKeyPair();
console.timeEnd('get key 2');
console.time('verify 2');
assert(!ecdsa.verify(msg, signature, keys.pub));
console.timeEnd('verify 2');

// Just a benchmark
console.time('verify 3');
for (var i = 0; i < 80; i++)
  assert(!ecdsa.verify(msg, signature, keys.pub));
console.timeEnd('verify 3');
